@@ -1,32 +1,38 @@
-import getpass
 import json
 import os
+from dataclasses import asdict
 
 import click
 import yaml
 from click.exceptions import ClickException
 from requests import ConnectionError
 
-from ethz_iam_webservice.utils import to_date
+from .ethz_iam import ETH_IAM
+from .group import RecertificationPeriod
+from .utils import gen_password
 
-from .ethz_iam import ETH_IAM_conn
-from .utils import (
-    check_password,
-    format_leitzahl,
-    format_notification,
-    gen_password,
-    to_date,
-)
 from .verbose import VERBOSE
 
+recertification_period_map = {
+    "A": "Annual",
+    "Y": "Annual",
+    "Q": "Quarterly",
+    "B": "Biennial",
+    "N": "No recertification",
+}
 
-class Service:
-    def __init__(self, conn, data):
-        self.conn = conn
-        self.data = data
-        if data:
-            for key in data:
-                setattr(self, key, data[key])
+
+class Credentials(object):
+    def __init__(self, username=None, password=None):
+        self.username = username or click.prompt(
+            text="Username",
+            default=os.environ.get("USER", ""),
+            show_default=True,
+        )
+        self.password = password or click.prompt(text="Password", hide_input=True)
+
+
+pass_iam_credentials = click.make_pass_decorator(Credentials)
 
 
 def _load_configuration(paths, filename=".ethz_iam_webservice"):
@@ -44,10 +50,8 @@ def _load_configuration(paths, filename=".ethz_iam_webservice"):
                     config = yaml.safe_load(stream)
                     for admin_account in config["admin_accounts"]:
                         admin_accounts.append(admin_account)
-                except yaml.YAMLError as e:
-
-                    print(e)
-                    return None
+                except yaml.YAMLError as exc:
+                    raise ClickException(exc)
 
     return admin_accounts
 
@@ -55,29 +59,11 @@ def _load_configuration(paths, filename=".ethz_iam_webservice"):
 def login(
     admin_username=None,
     admin_password=None,
-    hostname="https://iam.passwort.ethz.ch",
-    verify_certificates=True,
 ):
-    hostname = hostname
 
-    if admin_username is None:
-        admin_username = os.environ.get(
-            "IAM_USERNAME", input("Enter the admin username: ")
-        )
-
-    if admin_password is None:
-        admin_password = os.environ.get(
-            "IAM_PASSWORD",
-            getpass.getpass(
-                "Enter the password for admin user {}".format(admin_username)
-            ),
-        )
-
-    return ETH_IAM_conn(
+    return ETH_IAM(
         admin_username,
         admin_password,
-        hostname=hostname,
-        verify_certificates=verify_certificates,
     )
 
 
@@ -100,144 +86,228 @@ def get_username_password(ctx):
 @click.option(
     "-u",
     "--username",
-    help="username of your ETHZ IAM admin account (if not provided, it will be prompted)",
+    envvar="IAM_USERNAME",
+    help="username of ETHZ IAM admin account or IAM_USERNAME env",
 )
 @click.option(
     "--password",
-    help="password of your ETHZ IAM admin account (if not provided, it will be prompted)",
-)
-@click.option(
-    "-h", "--host", default="iam.passwort.ethz.ch", help="default: iam.passwort.ethz.ch"
+    envvar="IAM_PASSWORD",
+    help="password of ETHZ IAM admin account or IAM_PASSWORD env",
 )
+@click.version_option(prog_name="IAM command line tool")
 @click.pass_context
-def cli(ctx, host, username, password=None):
+def cli(ctx, username, password=None):
     """ETHZ IAM command-line tool."""
-    ctx.ensure_object(dict)
-    if not host.startswith("https://"):
-        host = "https://" + str(host)
-    ctx.obj["hostname"] = host
-    ctx.obj["username"] = username
-    ctx.obj["password"] = password
+    ctx.obj = Credentials(username, password)
 
 
 @cli.command("person", help="manage persons")
-@click.argument("person")
+@click.argument("identifier")
 @click.option(
     "-a",
     "--add-user",
     is_flag=True,
     help="Add an addtional user (aka persona) to this Person",
 )
+@click.option(
+    "--delete-user",
+    is_flag=True,
+    help="Delete a username (aka persona) of this Person",
+)
 @click.option("-u", "--username", help="Username")
-@click.option("-p", "--init-pw", help="Initial password")
+@click.option("-p", "--initpw", help="Initial password")
 @click.option("-m", "--mail", help="email address")
-@click.option("-g", "--given-name", help="firstname of this user")
-@click.option("-s", "--surname", help="surname of this user")
+@click.option("-g", "--firstname", help="given name of this user")
+@click.option("-s", "--familyname", help="surname of this user")
 @click.option("-d", "--description", help="Description about the purpose of this user")
-@click.pass_context
+@pass_iam_credentials
 def person(
-    ctx, person, add_user, username, init_pw, mail, given_name, surname, description
+    credentials,
+    identifier,
+    add_user,
+    delete_user,
+    username,
+    initpw,
+    mail,
+    firstname,
+    familyname,
+    description,
 ):
-    get_username_password(ctx)
 
-    e = login(ctx.obj["username"], ctx.obj["password"], ctx.obj["hostname"])
+    iam = login(credentials.username, credentials.password)
     try:
-        person = e.get_person(person)
+        person = iam.get_person(identifier)
     except ConnectionError as exc:
         raise ClickException(f"Cannot connect to IAM webservice at {exc.request.url}")
+    except ValueError as exc:
+        raise ClickException(f"No person found with this identifier: {identifier}")
 
     if add_user and username:
-        info = {
-            "username": username,
-            "password": init_pw,
-            "description": description,
-            "mail": mail,
-            "firstname": given_name or person["firstname"],
-            "familyname": surname or person["familyname"],
-        }
-        person.new_user(**info)
-        person = e.get_person(person)
+        try:
+            person.new_user(
+                username=username,
+                password=initpw,
+                description=description,
+                mail=mail,
+                firstname=firstname or person["firstname"],
+                familyname=familyname or person["familyname"],
+            )
+        except ValueError as exc:
+            raise ClickException(exc)
+        person = iam.get_person(identifier)
+    elif delete_user:
+        if not username:
+            raise ClickException(
+                "Please provide the username you want to delete with -u <username>"
+            )
+        iam.del_user(identifier=person.username, username=username)
+        person = iam.get_person(identifier)
 
     print(json.dumps(person.data, indent=4, sort_keys=True))
 
 
 @cli.command("group", help="manage security groups")
-@click.argument("group")
-@click.option("-d", "--delete", is_flag=True, help="delete this group")
+@click.argument("name")
 @click.option(
-    "-m",
-    "--members",
-    is_flag=True,
-    help="show members of the group",
-)
-@click.option(
-    "-i",
-    "--info",
-    is_flag=True,
-    default=True,
-    help="all information about the group",
-)
-@click.option(
-    "-a",
-    "--add",
-    help="username to add to group. Can be used multiple times: -a us1 -a us2",
-    multiple=True,
-)
-@click.option(
-    "-r",
-    "--remove",
-    help="username to remove from group. Can be used multiple times: -r us1 -r us2",
-    multiple=True,
-)
-@click.option(
-    "-n",
-    "--new",
-    is_flag=True,
-    help="create a new group",
-)
-@click.option(
-    "--agroup",
-    help="Admingroup for this group, required for new a group.",
+    "--new-name",
+    help="New name for this group (only used when renaming an existing group)",
 )
 @click.option(
     "-d",
     "--description",
     help="Description about this group.",
 )
+@click.option(
+    "--agroup",
+    "--ag",
+    help="Admingroup for this group, mandatory for new a group",
+)
 @click.option(
     "-t",
     "--target",
-    help="Target system for this group. Can be used multiple times: -t AD -t LDAPS",
+    help="Add target system for this group. Can be used multiple times: -t AD -t LDAP",
     multiple=True,
 )
-@click.pass_context
+@click.option(
+    "--remove-target",
+    "--rt",
+    help="Remove target system for this group. Can be used multiple times.",
+    multiple=True,
+)
+@click.option(
+    "--organizational-unit",
+    "--ou",
+    help="OU (organizational unit) for this group, e.g. AGRL, USYS, INFK etc. where this group should be stored. If not specified, this group will appear in OU=Custom,OU=ETHLists",
+)
+@click.option(
+    "--certification-period",
+    "--cp",
+    help="Define a certification period, whithin this group needs to be verified. [A]nnually, [B]iennial, [Q]uarterly, [N]one (default)",
+)
+@click.option(
+    "--certification-note",
+    "--cn",
+    help="Reason (certification note) in case you don't want to periodically certify this group",
+)
+@click.option(
+    "-m",
+    "--manager",
+    help="Username of the group manager for this group. Can appear multiple times. -m '' to remove all managers",
+    multiple=True,
+)
+@click.option(
+    "-a",
+    "--add",
+    help="Add username as member to group. Can be used multiple times: -a user1 -a user2",
+    multiple=True,
+)
+@click.option(
+    "-r",
+    "--remove",
+    help="Remove username as member to group. Can be used multiple times: -r user1 -r user2",
+    multiple=True,
+)
+@click.option(
+    "--add-subgroup",
+    "--as",
+    help="Add subgroup as member to group. Can be used multiple times.",
+    multiple=True,
+)
+@click.option(
+    "--remove-subgroup",
+    "--rs",
+    help="Remove subgroup as member from group. Can be used multiple times.",
+    multiple=True,
+)
+@click.option("--new", "-n", is_flag=True, help="Create a group")
+@click.option("--update", is_flag=True, help="Update a group")
+@click.option("--recertify", is_flag=True, help="Recertify a group")
+@click.option("--delete", is_flag=True, help="Delete a group")
+@pass_iam_credentials
 def group(
-    ctx,
-    group,
-    delete,
-    members,
-    info,
+    credentials,
+    name,
+    new_name=None,
+    description=None,
+    agroup=None,
+    target=None,
+    remove_target=None,
+    organizational_unit=None,
+    certification_period=None,
+    certification_note="No recertification needed",
+    manager=None,
     add=None,
     remove=None,
-    new=None,
-    agroup=None,
-    description=None,
-    target=None,
+    add_subgroup=None,
+    remove_subgroup=None,
+    new=False,
+    update=False,
+    recertify=False,
+    delete=False,
 ):
-    """group modifications."""
-    if add is None:
-        add = ()
-    if remove is None:
-        remove = ()
+    """manage groups
+    Name of the group must start with the admingroup's nickname,
+    followed by a dash, e.g. agrl-xxxx
+    """
+    iam = login(credentials.username, credentials.password)
+    try:
+        group = iam.get_group(name)
+    except ValueError:
+        group = None
+    except ConnectionError as exc:
+        raise ClickException(exc)
+
+    if certification_period:
+        if certification_period.upper() not in recertification_period_map:
+            raise ClickException(
+                "Please specify [A]nnual, [B]iennial, [Q]uarterly or [N]o recertification period."
+            )
+        else:
+            certification_period = recertification_period_map[
+                certification_period.upper()
+            ]
 
-    get_username_password(ctx)
-    e = login(ctx.obj["username"], ctx.obj["password"], ctx.obj["hostname"])
     if new:
+        if certification_period is None:
+            certification_period = RecertificationPeriod.NONE.value
+        if group:
+            raise ClickException(f"A group named {name} already exists.")
         if not agroup:
-            raise ClickException("Please provide an admin group with --agroup")
+            raise ClickException("Please provide an admingroup with --agroup")
+        if not description:
+            raise ClickException(
+                "Description of the group is missing. Use -d 'some description'"
+            )
         try:
-            group = e.new_group(
-                name=group, admingroup=agroup, description=description, targets=target
+            group = iam.new_group(
+                name=name,
+                description=description,
+                admingroup=agroup,
+                targets=[t.upper() for t in target],
+                group_ad_ou=organizational_unit,
+                certification_period=certification_period,
+                certification_note=certification_note,
+                managers=manager,
             )
         except ValueError as exc:
             raise ClickException(exc)
@@ -245,100 +315,127 @@ def group(
             raise ClickException(
                 f"Cannot connect to IAM webservice at {exc.request.url}"
             )
-    else:
-        try:
-            group = e.get_group(group)
-        except ValueError as exc:
-            raise ClickException(exc)
-        except ConnectionError as exc:
-            raise ClickException(
-                f"Cannot connect to IAM webservice at {exc.request.url}"
-            )
-
-    if add:
-        group.add_members(*add)
-    if remove:
-        group.del_members(*remove)
-
-    if add or remove or members:
-        print(json.dumps(group.members))
-        return
-
-    if delete:
-        click.confirm("Do you really want to delete this group?", abort=True)
+    elif delete:
         group.delete()
+        click.echo(f"Successfully deleted {name}")
         return
+    elif recertify:
+        group.recertify()
+        click.echo(f"Group {name} successfully recertified.")
+    elif update:
+        if not group:
+            raise ClickException(f"No group {name} exists.")
+        try:
+            new_group = group.update(
+                current_name=group.name,
+                new_name=new_name,
+                description=description,
+                group_ad_ou=organizational_unit,
+                certification_period=certification_period,
+                certification_note=certification_note,
+                managers=list(manager),
+            )
+            group = new_group if new_group else group
+        except ValueError as exc:
+            raise ClickException(exc)
 
-    print(json.dumps(group.data, indent=4, sort_keys=True))
+    if not group:
+        raise ClickException(
+            f"No group found with name {name}. Use --new if you want to create a new group."
+        )
+
+    if add or add_subgroup:
+        group.add_members(users=add, subgroups=add_subgroup)
+    if remove or remove_subgroup:
+        group.remove_members(users=remove, subgroups=remove_subgroup)
+    if target and not new:
+        targets = [t.upper() for t in target]
+        try:
+            group.set_targets(targets)
+        except ValueError as exc:
+            if "already present" in str(exc):
+                pass
+            else:
+                raise ClickException(exc)
+    if remove_target:
+        targets = [t.upper() for t in remove_target]
+        try:
+            group.remove_targets(targets)
+        except ValueError as exc:
+            if "already not present" in str(exc):
+                pass
+            else:
+                raise ClickException(exc)
+
+    print(json.dumps(asdict(group), indent=4, sort_keys=True))
 
 
 @cli.group("guest", help="manage guests")
-@click.pass_context
-def guest(ctx):
-    get_username_password(ctx)
-    iam = login(ctx.obj["username"], ctx.obj["password"], ctx.obj["hostname"])
-    ctx.obj["iam"] = iam
+def guest():
+    pass
 
 
 @guest.command("list", help="return all guest users of a given host leitzahl")
 @click.argument("leitzahl")
-@click.pass_context
-def get_guests(ctx, leitzahl):
-    iam = ctx.obj["iam"]
+@pass_iam_credentials
+def get_guests(credentials, leitzahl):
+    iam = login(credentials.username, credentials.password)
     try:
-        guests = iam.get_users_of_lz(leitzahl)
+        guests = iam.get_guests_of_lz(leitzahl)
     except Exception as exc:
         raise ClickException(exc)
-    click.echo(json.dumps(guests, indent=4, sort_keys=True))
+    click.echo(
+        json.dumps([asdict(guest) for guest in guests], indent=4, sort_keys=True)
+    )
 
 
 @guest.command("get", help="get an existing guest")
 @click.argument("username")
-@click.pass_context
-def get_guest(ctx, username):
-    iam = ctx.obj["iam"]
+@pass_iam_credentials
+def get_guest(credentials, username):
+    iam = login(credentials.username, credentials.password)
     try:
         guest = iam.get_guest(username)
     except ConnectionError as exc:
         raise ClickException(f"Cannot connect to IAM webservice at {exc.request.url}")
-    print(json.dumps(guest._data_formatted(), indent=4, sort_keys=True))
+    print(json.dumps(guest.data, indent=4, sort_keys=True))
 
 
 @guest.command(
     "extend", help="extend validation of an existing guest. Default is today+1 year."
 )
 @click.option(
-    "-e", "--endDate", help="specify end date of guest (YYYY-MM-DD or DD.MM.YYYY)."
+    "-e", "--end-date", help="specify end date of guest (YYYY-MM-DD or DD.MM.YYYY)."
 )
 @click.option(
     "-m", "--months", help="extend validation of an existing guest by this many months."
 )
 @click.argument("username")
-@click.pass_context
-def extend_guest(ctx, enddate, months, username):
-    iam = ctx.obj["iam"]
+@pass_iam_credentials
+def extend_guest(credentials, end_date, months, username):
+    iam = login(credentials.username, credentials.password)
     try:
-        guest = iam.extend_guest(username, enddate, months)
-        print(json.dumps(guest._data_formatted(), indent=4, sort_keys=True))
+        guest = iam.extend_guest(username=username, end_date=end_date, months=months)
+        print(json.dumps(guest.data, indent=4, sort_keys=True))
     except Exception as exc:
         raise ClickException(exc)
 
 
 @click.option("-d", "--description", help="")
-@click.option("-h", "--host", help="ETHZ Username of host")
+@click.option("-h", "--host-username", help="ETHZ Username of host")
 @click.option(
     "-a",
-    "--adminGroup",
+    "--host-admingroup",
     help="Name of the admin group this guest should be connected to. Default: same as the technical user which is creating this guest.",
 )
 @click.option(
-    "-o",
-    "--hostOrg",
+    "-l",
+    "--host-leitzahl",
     help="Leitzahl of host organization, see org.ethz.ch. Default: Leitzahl of the host.",
 )
 @click.option(
     "-c",
-    "--technicalContact",
+    "--technical-contact",
     help="email address of technical contact. Default: email of the host of this guest.",
 )
 @click.option(
@@ -346,93 +443,80 @@ def extend_guest(ctx, enddate, months, username):
     "--notification",
     help="g=guest, h=host, t=technical contact. Use any combination of the 3 chars. Defaults to «gh»",
 )
-@click.option("-m", "--email", "--mail", help="Change email address of the guest")
 @click.option(
-    "-s", "--startDate", help="Start date of guest (YYYY-DD-MM). Default: today"
+    "-e", "--end-date", help="End date of guest (YYYY-MM-DD). Default: today+1 year"
 )
 @click.option(
-    "-e", "--endDate", help="End date of guest (YYYY-MM-DD). Default: today+1 year"
-)
-@click.option(
-    "--deactivationStartDate",
+    "--deactivation-start-date",
     help='Deactivation start date of guest (YYYY-DD-MM). Set it to "" to remove',
 )
 @click.option(
-    "--deactivationEndDate",
+    "--deactivation-end-date",
     help='Deactivation end date of guest (YYYY-MM-DD). Set it to "" to remove',
 )
 @guest.command("update", help="update an existing guest")
 @click.argument("username")
-@click.pass_context
+@pass_iam_credentials
 def update_guest(
-    ctx,
+    credentials,
     description,
-    hostorg,
-    host,
-    technicalcontact,
-    admingroup,
+    host_leitzahl,
+    host_username,
+    technical_contact,
+    host_admingroup,
     notification,
-    email,
-    startdate,
-    enddate,
+    end_date,
     username,
-    deactivationstartdate,
-    deactivationenddate,
+    deactivation_start_date,
+    deactivation_end_date,
 ):
-    if startdate:
-        startdate = to_date(startdate)
-    if enddate:
-        enddate = to_date(enddate)
-    if deactivationstartdate:
-        deactivationstartdate = to_date(deactivationstartdate)
-    if deactivationenddate:
-        deactivationenddate = to_date(deactivationenddate)
 
-    iam = ctx.obj["iam"]
+    iam = login(credentials.username, credentials.password)
     try:
         guest = iam.update_guest(
             username=username,
-            host=host,
-            respAdminRole=admingroup,
+            host_username=host_username,
+            host_admingroup=host_admingroup,
             description=description,
-            guestTechnicalContact=technicalcontact,
+            technical_contact=technical_contact,
             notification=notification,
-            hostOrg=hostorg,
-            startDate=startdate,
-            endDate=enddate,
-            deactivationStartDate=deactivationstartdate,
-            deactivationEndDate=deactivationenddate,
+            host_leitzahl=host_leitzahl,
+            end_date=end_date,
+            deactivation_start_date=deactivation_start_date,
+            deactivation_end_date=deactivation_end_date,
         )
-        print(json.dumps(guest._data_formatted(), indent=4, sort_keys=True))
+        print(json.dumps(guest.data, indent=4, sort_keys=True))
     except ConnectionError as exc:
         raise ClickException(f"Cannot connect to IAM webservice at {exc.request.url}")
 
 
 @guest.command("new", help="create a new guest")
-@click.option("-f", "--firstName", required=True, help="given name")
-@click.option("-l", "--lastName", required=True, help="surname")
+@click.option("-g", "--firstname", required=True, help="given name")
+@click.option("-s", "--familyname", required=True, help="surname")
 @click.option("-m", "--mail", required=True, help="email address")
 @click.option("-d", "--description", required=True, help="")
-@click.option("-h", "--host", required=True, help="ETHZ Username of host")
+@click.option("-h", "--host-username", required=True, help="ETHZ Username of host")
 @click.option(
     "-a",
-    "--adminGroup",
+    "--host-admingroup",
     required=True,
     help="Name of the admin group this guest should be connected to. Default: same as the technical user which is creating this guest.",
 )
 @click.option(
     "-b",
-    "--dateOfBirth",
+    "--birth-date",
     help="birthdate in YYYY-MM-DD format. Default: Today's date + year 2000",
 )
 @click.option(
-    "-o",
-    "--hostOrg",
+    "-l",
+    "--host-leitzahl",
+    required=True,
     help="Leitzahl of host organization, see org.ethz.ch. Default: Leitzahl of the host.",
 )
 @click.option(
     "-c",
-    "--technicalContact",
+    "--technical-contact",
+    required=True,
     help="email address of technical contact. Default: email of the host of this guest.",
 )
 @click.option(
@@ -442,77 +526,75 @@ def update_guest(
     help="g=guest, h=host, t=technical contact. Use any combination of the 3 chars. ",
 )
 @click.option(
-    "-s", "--startDate", help="Start date of guest (YYYY-DD-MM). Default: today"
+    "--sd", "--start-date", help="Start date of guest (YYYY-DD-MM). Default: today"
 )
 @click.option(
-    "-e", "--endDate", help="End date of guest (YYYY-MM-DD). Default: today+1 year"
+    "--ed", "--end-date", help="End date of guest (YYYY-MM-DD). Default: today+1 year"
 )
-@click.option("--no-initpw", is_flag=True, help="do not change the initial password")
-@click.pass_context
+@click.option(
+    "--init-password",
+    is_flag=True,
+    help="Set initial password and return it in cleartext",
+)
+@pass_iam_credentials
 def new_guest(
-    ctx,
+    credentials,
     firstname,
-    lastname,
+    familyname,
     mail,
     description,
-    dateofbirth,
-    hostorg,
-    host,
-    technicalcontact,
-    admingroup,
+    birth_date,
+    host_leitzahl,
+    host_username,
+    technical_contact,
+    host_admingroup,
     notification,
-    startdate,
-    enddate,
-    no_initpw,
+    sd,
+    ed,
+    init_password,
 ):
-    if dateofbirth:
-        dateofbirth = to_date(dateofbirth)
-    if startdate:
-        startdate = to_date(startdate)
-    if enddate:
-        enddate = to_date(enddate)
-
-    iam = ctx.obj["iam"]
+    iam = login(credentials.username, credentials.password)
     try:
+        pass
         guest = iam.new_guest(
-            firstName=firstname,
-            lastName=lastname,
+            firstname=firstname,
+            familyname=familyname,
             mail=mail,
-            host=host,
-            respAdminRole=admingroup,
+            host_username=host_username,
+            host_admingroup=host_admingroup,
             description=description,
-            guestTechnicalContact=technicalcontact,
+            technical_contact=technical_contact,
             notification=notification,
-            hostOrg=hostorg,
-            startDate=startdate,
-            endDate=enddate,
+            host_leitzahl=host_leitzahl,
+            start_date=sd,
+            end_date=ed,
         )
-        guest.save()
     except ConnectionError as exc:
         raise ClickException(f"Cannot connect to IAM webservice at {exc.request.url}")
 
     ldap_password = gen_password()
     vpn_password = gen_password()
-    guest_data = guest._data_formatted()
-    guest_username = guest_data.get("username")
+    guest_data = guest.data
 
-    if not guest_username:
+    if not guest.username:
         ClickException(
             "could not set inital password for the guest because the guest has no username."
         )
 
-    if not no_initpw:
-        guest_user = iam.get_user(guest_username)
+    if init_password:
+        guest_user = iam.get_user(guest.username)
         try:
             guest_user.set_password(password=ldap_password, service_name="LDAP")
             guest_data["init_ldap_password"] = ldap_password
-        except Exception:
+        except Exception as exc:
+            print(exc)
             pass
 
         try:
             guest_user.set_password(password=vpn_password, service_name="VPN")
             guest_data["init_vpn_password"] = vpn_password
         except Exception as exc:
+            print(exc)
             pass
 
     print(json.dumps(guest_data, indent=4, sort_keys=True))
@@ -560,9 +642,9 @@ def new_guest(
     "--service-password",
     help="set a password for the given service. Use the --service option to specify the service.",
 )
-@click.pass_context
+@pass_iam_credentials
 def user(
-    ctx,
+    credentials,
     username,
     delete,
     info,
@@ -573,10 +655,9 @@ def user(
     service_password=None,
     service=None,
 ):
-    get_username_password(ctx)
-    e = login(ctx.obj["username"], ctx.obj["password"], ctx.obj["hostname"])
+    iam = login(credentials.username, credentials.password)
     try:
-        user = e.get_user(username)
+        user = iam.get_user(username)
     except ConnectionError as exc:
         raise ClickException(f"Cannot connect to IAM webservice at {exc.request.url}")
 
